clear all;

gamma = 1.0;
sigma = 0.05;

gamma2 = gamma^2;
sigma2 = sigma^2;
sigmam2 = sigma^(-2);

sigmar = 1/sigma;

M = 101;
xmin = 0;
xmax = 100;
L = xmax-xmin;
Dx = L/(M-1);
xm = xmin + (xmax-xmin)*[0:M-1]'/(M-1);
r = 0.0001;
rr=r;

% true model parameters sinusoidal
ptrue = 5*pi/L;
fprintf('p %f\n',ptrue);
m = sin( ptrue*xm);

% prior model parameters
mpri = m;

% guess for p
p = 0.5*ptrue;

% data are some of the model parameters
% with additive random noise
N = 40;
k = sort(unidrnd(M,N,1));
N = length(k);
xd = xm(k);
d = m(k)+random('Normal',0,sigma,N,1);

figure(1);
clf;
hold on;
set(gca,'LineWidth',2);
set(gca,'FontSize',14);
plot( xm, m, 'k-', 'LineWidth', 6);
xlabel('x');
ylabel('m(x) and d(x)');

% selects data d from model paramerts m
G = zeros(N,M);
for i=[1:N]
    G(i,k(i)) = 1;
end

A = zeros(N,N);
Cdd = zeros(N,N);
dAdp = zeros(N,N);
dCdddp = zeros(N,N);
for i=[1:N]
for j=[1:N]
    Cdd(i,j) = gamma2 * cos( p*(xd(i)-xd(j)) ) * exp( -r*abs(xd(i)-xd(j)) );
    A(i,j) = Cdd(i,j) + sigma2*(i==j);
    dCdddp(i,j) = -gamma2 * (xd(i)-xd(j)) * sin(p*(xd(i)-xd(j))) * exp( -r*abs(xd(i)-xd(j)) );
    dAdp(i,j) = dCdddp(i,j);
end
end

Cmd = zeros(M,N);
dCmddp = zeros(M,N);
for i=[1:M]
for j=[1:N]
    Cmd(i,j) = gamma2 * cos( p*(xm(i)-xd(j)) ) * exp( -r*abs(xm(i)-xd(j)) );
    dCmddp(i,j) = -gamma2 * (xm(i)-xd(j)) * sin(p*(xm(i)-xd(j))) * exp( -r*abs(xm(i)-xd(j)) );
end
end

Cmm = zeros(M,M);
dCmmdp = zeros(M,M);
for i=[1:M]
for j=[1:M]
    Cmm(i,j) = gamma2 * cos( p*(xm(i)-xm(j)) ) * exp( -rr*abs(xm(i)-xm(j)) );
    dCmmdp(i,j) = -gamma2 * (xm(i)-xm(j)) * sin(p*(xm(i)-xm(j))) * exp(-rr*abs(xm(i)-xm(j)));
end
end
CmmI = inv(Cmm);
CmmIsr = sqrtm(CmmI);
dCmmIdp = - CmmI * dCmmdp * CmmI;
dCmmIsrdp = sylvester( CmmIsr, CmmIsr, dCmmIdp );
R = chol(Cmm);
logdetCmm = sum(log(diag(R)));

% perturbation in p
Dp = 1e-6;
pp = p+Dp;

% perturbed quantities, signified by suffix Z
AZ = zeros(N,N);
CddZ = zeros(N,N);
for i=[1:N]
for j=[1:N]
    CddZ(i,j) = gamma2 * cos( pp*(xd(i)-xd(j)) ) * exp( -r*abs(xd(i)-xd(j)) );
    AZ(i,j) = CddZ(i,j) + sigma2*(i==j);
end
end

CmdZ = zeros(M,N);
for i=[1:M]
for j=[1:N]
    CmdZ(i,j) = gamma2 * cos( pp*(xm(i)-xd(j)) ) * exp( -r*abs(xm(i)-xd(j)) );
end
end

CmmZ = zeros(M,M);
dCmmdpZ = zeros(M,M);
for i=[1:M]
for j=[1:M]
    CmmZ(i,j) = gamma2 * cos( pp*(xm(i)-xm(j)) ) * exp( -rr*abs(xm(i)-xm(j)) );
    dCmmdpZ(i,j) = -gamma2 * (xm(i)-xm(j)) * sin(pp*(xm(i)-xm(j))) * exp(-rr*abs(xm(i)-xm(j)));
end
end
CmmIZ = inv(CmmZ);
CmmIsrZ = sqrtm(CmmIZ);
dCmmIdpZ = - CmmIZ * dCmmdpZ * CmmIZ;
dCmmIsrdpZ = sylvester( CmmIsrZ, CmmIsrZ, dCmmIdpZ );

% numerical estimate of dAdp
dAdp_num = (AZ-A)/Dp;
E = sum(sum((dAdp-dAdp_num).^2))/(N^2);
V = sum(sum((dAdp).^2))/(N^2);
fprintf('relative error in dAdp: %e\n',E/V);

% solution
u = A\d;
mest = Cmd*u;
dpre = G*mest;
e = d - dpre;
ET = sigmam2*(e'*e);
plot( xm, mest, 'g-', 'LineWidth', 4);

% derivative of predicted data
v = A\(dAdp*u);
dmestdp = dCmddp*u - Cmd*v;
ddpredp = G*dCmddp*u - G*Cmd*v;

% derivative of the error
dedp = -ddpredp;

% derivative of error E fby first formula
dETdp = sigmam2*(-2*e'*G*dCmddp*u + 2*e'*G*Cmd*v);

% derivative of error E by second formula
b = A\(dCmddp'*G'*e);
c = A\(Cmd'*G'*e);
dETdp_b = sigmam2*(-2*b'*d + 2*c'*dCdddp*u);

% by fiffernetiating dot product
dETdp_c = 2*sigmam2*e'*dedp;

% solution of perturbed problem
uZ = AZ\d;
mestZ = CmdZ*uZ;
dmestdp_num = (mestZ-mest)/Dp;
dpreZ = G*mestZ;
eZ = d - dpreZ;
ETZ = sigmam2*(eZ'*eZ);

% numerical estimate of ddpredp
ddpredp_num = (dpreZ-dpre)/Dp;
E2 = sum((ddpredp-ddpredp_num).^2);
V2 = sum((ddpredp).^2);
fprintf('relative error in ddpredp: %e\n',E2/V2);

% numerical estimate of dEdp
dETdp_num = (ETZ-ET)/Dp;
fprintf('ddpredp: formula_1 %e formula_2 %e formula_3 %e numerical %e\n',dETdp, dETdp_b, dETdp_c, dETdp_num);

% prior error and its derivative
el=(mpri-mest);
elZ=(mpri-mestZ);
eltilde = CmmIsr*el;
eltildeZ = CmmIsrZ*elZ;
deltildedp = dCmmIsrdp*el - CmmIsr*dmestdp;

% total prior error and its derivative
LT = eltilde'*eltilde;
LTZ = eltildeZ'*eltildeZ;
dLTdp = 2*eltilde'*deltildedp;
dLTdp_num = (LTZ-LT)/Dp;
fprintf('dLTdp: analtyic %e numerical %e\n',dLTdp, dLTdp_num);

PSI = logdetCmm + ET + LT;

figure(2);
clf;
hold on;
set(gca,'LineWidth',2);
set(gca,'FontSize',14);
plot( xm, dmestdp, 'k-', 'LineWidth', 5);
plot( xm, dmestdp_num, 'y-', 'LineWidth', 2);
plot( xd, ddpredp, 'k*', 'LineWidth', 2);
plot( xd, ddpredp_num, 'yo', 'LineWidth', 2);
xlabel('x');
ylabel('dmdp and dd/dp');

% Newton's Method adjustment of p

pg = p;
Dp = 0.01;
NITER = 30;
pg = pg-Dp;  % so first iteration starts pg
alpha = 1.0;
nu = 1.0;
for iter = [1:NITER]
    pg = pg+nu*alpha*Dp; % increment
    
    A = zeros(N,N);
    Cdd = zeros(N,N);
    dAdp = zeros(N,N);
    dCdddp = zeros(N,N);
    pg2 = pg^2;
    for i=[1:N]
    for j=[1:N]
        Cdd(i,j) = gamma2 * cos( pg*(xd(i)-xd(j)) ) * exp( -rr*abs(xd(i)-xd(j)) );
        A(i,j) = Cdd(i,j) + sigma2*(i==j);
        dCdddp(i,j) = -gamma2 * (xd(i)-xd(j)) * sin(pg*(xd(i)-xd(j))) * exp(-rr*abs(xd(i)-xd(j)));
        dAdp(i,j) = dCdddp(i,j);
    end
    end

    Cmd = zeros(M,N);
    dCmddp = zeros(M,N);
    for i=[1:M]
    for j=[1:N]
        Cmd(i,j) = gamma2 * cos( pg*(xm(i)-xd(j)) ) * exp( -rr*abs(xm(i)-xd(j)) );
        dCmddp(i,j) = -gamma2 * (xm(i)-xd(j)) * sin(pg*(xm(i)-xd(j))) * exp(-rr*abs(xm(i)-xd(j)));
    end
    end
    
    Cmm = zeros(M,M);
    dCmmdp = zeros(M,M);
    for i=[1:M]
    for j=[1:M]
        Cmm(i,j) = gamma2 * cos( pg*(xm(i)-xm(j)) ) * exp( -rr*abs(xm(i)-xm(j)) );
        dCmmdp(i,j) = -gamma2 * (xm(i)-xm(j)) * sin(pg*(xm(i)-xm(j))) * exp(-rr*abs(xm(i)-xm(j)));
    end
    end
    CmmI = inv(Cmm);
    CmmIsr = sqrtm(CmmI);
    dCmmIdp = - CmmI * dCmmdp * CmmI;
    dCmmIsrdp = sylvester( CmmIsr, CmmIsr, dCmmIdp );
    R = chol(Cmm);
    logdetCmm = sum(log(diag(R)));
    dlogdetCmmdp = trace( CmmI*dCmmdp );

    % solution
    u = A\d;
    mg = Cmd*u;
    dg = G*mg;

    % derivative of predicted data
    v = A\(dAdp*u);
    dmestdp = dCmddp*u - Cmd*v;
    ddpredp = G*dCmddp*u - G*Cmd*v;
    detildedp = -sigmar*ddpredp;
    e = d-dg;
    etilde = sigmar*e;
    
    % derivative of total data error
    E = etilde'*etilde;
    dEdp = 2*etilde'*detildedp;
    
    % total data error and its derivative
    el=(mpri-mg);
    eltilde = CmmIsr*el;
    deltildedq = dCmmIsrdp*el - CmmIsr*dmestdp;
    
    % total prior error and its derivative
    L = eltilde'*eltilde;
    dLdp = 2*eltilde'*deltildedq;
    
    PSI = logdetCmm + E + L;
    dPSIdp = dlogdetCmmdp + dEdp + dLdp;
    nu = -sign(dPSIdp);  % downhill vector
   
    % gradient method method
    if( iter==1 )  % on the first iteration do nothing but save
        pglast = pg;
        PSIlast = PSI;
        fprintf('1: pg %e nu %e alpha %e PSI %e PSIlast %e\n', pg, nu, alpha, PSI, PSIlast);
    elseif (PSI < PSIlast )
        pglast = pg;
        PSIlast = PSI;
        fprintf('2: pg %e nu %e alpha %e PSI %e PSilast %e\n', pg, nu, alpha, PSI, PSIlast);
    else % psi increased, so decrease alpha
        pg = pglast;
        PSI = PSIlast;
        alpha = alpha/2;
        fprintf('2: pg %e nu %e alpha %e PSI %e PSIlast %e\n', pg, nu, alpha, PSI, PSIlast);
    end
        
end
pest = pg;

figure(1);
plot( xm, mg, 'y-', 'LineWidth', 2);
plot( xd, d, 'ro', 'LineWidth', 2);

fprintf('ptrue %e pest %e relating error %e\n', ptrue, pest, (ptrue-pest)/ptrue );




