clear all;

% standard set up
N=256;
Dt=1.0;
tmax=Dt*(N-1); 
t=Dt*[0:N-1]'; 
fmax=1/(2.0*Dt);
df=fmax/(N/2);
f=df*[0:N/2,-N/2+1:-1]'; 
Nf=N/2+1;
dw=2*pi*df;
w=dw*[0:N/2,-N/2+1:-1]';
Nw=Nf;

% two-sided decaying exponential autocorrelation function
tmid=tmax/2;
s = 20/tmax;
c = exp( -s*abs(t-tmid) );

figure(1);
clf;
subplot(1,2,1);
hold on;
set(gca,'LineWidth',2);
set(gca,'FontSize', 14);
axis( [ -tmid, tmid, -0.5, 1.5] );
xlabel('time lag t (s)');
ylabel('C(t)');
plot(t-tmid,c,'k-','LineWidth',3);

ct = fft(c);
asd = abs(ct);

figure(2);
clf;
subplot(1,2,1);
hold on;
set(gca,'LineWidth',2);
set(gca,'FontSize', 14);
axis( [ 0, fmax, 0, 15] );
xlabel('frequency f (Hz)');
ylabel('c(f)');
plot(f,asd,'k-','LineWidth',3);

d = random('normal',0,1,N,1);
d = d - mean(d);
dt = fft(d);
dasd = abs(dt);
dt = asd.*dt./dasd;
dtrue = real(ifft(dt));
vard = var(dtrue);
dtrue = dtrue/sqrt(vard);
dasd = abs(fft(dtrue));

figure(3);
clf;
subplot(2,1,1);
set(gca,'LineWidth',2);
set(gca,'FontSize', 14);
axis( [ 0, N, -5, 5] );
xlabel('time t (s)');
ylabel('d(t)');
hold on;
plot(t,dtrue,'k','LineWidth',3);


% gaussian autocorrelation function
tmid=tmax/2;
s = 20/tmax;
s = 40/tmax;
c = exp( -0.5 * (s^2) *abs(t-tmid).^2 );

figure(1);
subplot(1,2,2);
hold on;
set(gca,'LineWidth',2);
set(gca,'FontSize', 14);
axis( [ -tmid, tmid, -0.5, 1.5] );
xlabel('time lag t (s)');
ylabel('C(t)');
plot(t-tmid,c,'k-','LineWidth',3);

ct = fft(c);
asd = abs(ct);

figure(2);
subplot(1,2,2);
hold on;
set(gca,'LineWidth',2);
set(gca,'FontSize', 14);
axis( [ 0, fmax, 0, 15] );
xlabel('frequency f (Hz)');
ylabel('c(f)');
plot(f,asd,'k-','LineWidth',3);

d = random('normal',0,1,N,1);
d = d - mean(d);
dt = fft(d);
dasd = abs(dt);
dt = asd.*dt./dasd;
dtrue = real(ifft(dt));
vard = var(dtrue);
dtrue = dtrue/sqrt(vard);
dasd = abs(fft(dtrue));

figure(3);
subplot(2,1,2);
set(gca,'LineWidth',2);
set(gca,'FontSize', 14);
axis( [ 0, N, -5, 5] );
xlabel('time t (s)');
ylabel('d(t)');
hold on;
plot(t,dtrue,'k','LineWidth',3);
