function mu = eda_bradley_fayyad_mean(S,K,Nsets,fraction)
% create initial k-mean cluster means, by
% the Bradley & Fayyad (1998) method, which
% is based on clustering clusters. The NxM
% sample matrix S is randomly subsampled Nsets
% times, with each set containing (fraction*N+2*K)
% samples.
% returns KxM matrix mu of means

    [N,M] = size(S);
    initmu = eda_random_partition_mean(S,K);
    KiMAX = K*Nsets;  % maximum number of means mu
    Ni = floor(fraction*N)+2*K; % size of small sets

    % Step 1, create KixM table CM of means of subsampled data
    CM = zeros(KiMAX,M);
    Ki=1; % counter for rows of CM
    for j=[1:Nsets]
        k = unidrnd(N,Ni,1);
        Si = S(k,1:M);
        [mymu, myk, mydist2, mycount, myKdist2] = eda_kmeans_mod(Si, initmu);
        CM(Ki:Ki+K-1,1:M) = mymu(1:K,1:M);
        Ki=Ki+K;
    end

    % Step 2, create KixM table FMS of means of cluster means
    FMS = zeros(KiMAX,M);
    Ki=1; % counter for rows of CM
    for j=[1:Nsets]
        [mymu, myk, mydist2, mycount, myKdist2] = eda_kmeans(CM, CM(Ki:Ki+K-1,1:M));
        FMS(Ki:Ki+K-1,1:M) = mymu(1:K,1:M);
        Ki=Ki+K;
    end

    % Step 3A: associate each KxM member FMi of Fm
    %          with all individual 1xM elements of CM
    %          and compute the overall error
    error = zeros(Nsets,1);
    Ki=1;
    for i=[1:Nsets]
        FMi = FMS(Ki:Ki+K-1,1:M); % one group of means
        Ki=Ki+K;
        for j=[1:KiMAX]
            CMj = CM(j,1:M); % one member of CM
            for m=[1:K]
                delta = (CMj(1:M) - FMi(m,1:M))';
                r2 = delta'*delta;
                if( m==1 ) % always accept first distance
                    r2min = r2;
                elseif r2<r2min % accept if smaller
                    r2min = r2;
                end
            end
        end
        error(i) = error(i) + r2min;
    end
    
    % Step 3B: select the FMi with lowest error
    [tmp, i] = min(error);
    mu = FMS(K*(i-1)+1:K*(i-1)+K,1:M);
end

