function [newmu,k,dist2,count,Kdist2] = eda_kmeans(S, mu)
% k-mean cluster analsis of NxM sample natrix S
% starting from initial guess KxM mu of cluster means
% returns:
% newmu: new cluster means
% k: Nx1 vector of cluster assignments
% dist2: Nx1 vector of squared distance to cluster center
% count: Kx1 vector of samples in cluster
% Kdist2: KxK matrix of inter-cluster squared distances
    [N, M] = size(S);   % N samples with M elements
    [K, M2] = size(mu); % K cluster meabs
    k = zeros(N,1);     % assoc sample w cluster
    dist2 = zeros(N,1);  % sq distance from cluster mean
    Kdist2 = zeros(K,K); % cluster to nearest cluster sq distance
    oldmu = mu;  % old cluster means
    
    MAXP = 2*K*M;  % max iterations
    for p=[1:MAXP]  % iterate
  
        % cluster assignment step
        changes = 0; % number of changes
        for i=[1:N] % loop over samples
            for j=[1:K] % loop over cluster means
                % deviation of sample from mean
                delta = (S(i,1:M) - oldmu(j,1:M))';
                % squared distance
                r2 = delta'*delta;
                if( j==1 ) % always accept first distance
                    r2min = r2;
                    newk = j;
                elseif r2<r2min % accept if smaller
                    r2min = r2;
                    newk = j;
                end
            end
            if k(i) ~= newk % count reassignments
                changes=changes+1;
            end
            k(i) = newk;  % reassigned cluster
            dist2(i) = r2min; % sq distance to that cluster
        end

        % end loop over iterations on no further changes
        if( (p~=1) && (changes==0) )
            break;
        end

        % means step
        newmu = zeros(K,M);  % new cluster means
        count = zeros(K,1);  % samples in cluster
        for i=[1:N]   % loop over samples
            for j=[1:M] % loop over elements
                % cluster sum
                newmu(k(i),j) = newmu(k(i),j) + S(i,j);
            end
            % count samples in cluster
            count(k(i)) = count(k(i)) + 1;
        end
        for j=[1:K] % loop over clusters
            if( count(j) == 0 ) % cluster with no samples
                % no change in cluster mean
                newmu(j,1:M) = oldmu(j,1:M);
            else
                % new cluster mean
                newmu(j,1:M) = newmu(j,1:M)/count(j);
            end
        end
        oldmu=newmu; % update cluster mean
    end
        
    % inter-cluster distances
    for i=[1:K-1]    % loop over cluster means
        for j=[2:K]  % loop over cluster means
            delta = (newmu(i,1:M) - newmu(j,1:M))';
            r2 = delta'*delta;
            Kdist2(i,j) = r2; % sq distance between clusters
            Kdist2(j,i) = r2; % sq distance between clusters
        end
    end
end

