function [ tBmA_est, Cmax_est ] = timedelay( uA, uB, Dt )
% time delay between two similar timeseries, by
% cross-correlation followed by parabilic refinment.
% delay is positive when pulse on B is later than pulse on A
% input:
% uA, uB, two signals to be compared
%         should have zero mean
%         if of unequal length, the shorter is zero padded
% Dt, sampling of both signals
% output
% tBmA_est, estimated time delay
% Cmax_est, cross-correlation at estimated delay


c = xcorr(uA, uB);        % cross-correlate
NA = length(uA);          % length of u
NB = length(uB);          % length of v
Nm = max([NA,NB]);        % max of two lengths
Nc = length(c);           % length of cross-correlation

% lag to nearest sample
[cmax, icmax] = max(c);   % determine location of maximum
tBmArough = -Dt * (icmax-Nm);  % time lag

% parabolic refinement
% (c-cmax) = m(1) + m(2) dt + m(3) dt^2
% d(c-cmax)/ddt = 0 = m(2) + 2 m(3) dt
% dt = -0.5*m(2)/m(3)
% c = cmax + m(1) + m(2) dt + m(3) dt^2
G = [ [1, 1, 1]', Dt*[-1, 0, 1]', (Dt^2)*[1, 0, 1]' ];
d = [ c(icmax-1)-cmax, c(icmax)-cmax, c(icmax+1)-cmax ]';
m = (G'*G)\(G'*d);
dt = -0.5*m(2)/m(3);
Cmax_est = cmax + m(1) + m(2)*dt + m(3)*(dt^2);
tBmA_est = tBmArough - dt;


end


